/*
 * ttwrap.c
 *
 * Wrappers for the translation table module
 *
 */
#include "parti.h"
#include <stdio.h>

TTABLE *get_table_ptr();


int build_reg_translation_table_ ( dist_type, data_size )
     intPtr  dist_type, data_size;
{
  return(register_table(build_reg_translation_table(*dist_type,*data_size)));
}




int build_dst_translation_table_ ( index, nindex, repf, psize )
     intPtr index, nindex, psize;
     float *repf;
{
  int i, result;

  for (i=0;i<*nindex;i++) index[i]--;
  result = register_table(mkDistTable(index, *nindex, *repf, *psize));
  for (i=0;i<*nindex;i++) index[i]++;
  
  return(result);
}



int init_ttable_with_proc_(type,color,ndata)
     int *type,*ndata,*color;
{
  TTABLE  *trans_table, *init_ttable_with_proc();

  trans_table = init_ttable_with_proc(*type, color, *ndata);
  return(register_table(trans_table));
}


int build_translation_table_ ( type,index_array,nindex)
     int *type,*nindex,*index_array;
{
  int i, result;

  for (i=0;i<*nindex;i++) index_array[i]--;
  result = register_table(build_translation_table(*type,index_array,*nindex));
  for (i=0;i<*nindex;i++) index_array[i]++;

  return(result);
}




void    free_table_(trans_table)
     intPtr trans_table;
{
  free_table(get_table_ptr(*trans_table));
}



void dereference_(trans_table,index_array,local,proc,ndata)
     intPtr trans_table, index_array, local,proc, ndata;
{ 
  int i;

  for (i=0;i<*ndata;i++) index_array[i]--; 
  _dereference(get_table_ptr(*trans_table),index_array,local,proc,*ndata,OFF);
  for (i=0;i<*ndata;i++) index_array[i]++,local[i]++; 
}



void dereference(trans_table,index_array,local,proc,ndata)
   TTABLE *trans_table;
   intPtr index_array, local, proc;
   int ndata;
{ 
  _dereference(trans_table,index_array,local,proc,ndata,OFF);
}


void derefproc_(trans_table,index_array,proc,ndata)
     intPtr trans_table, index_array, proc, ndata;
{ 
  int i;

  for (i=0;i<*ndata;i++) index_array[i]--; 
  derefproc(get_table_ptr(*trans_table),index_array,proc,*ndata,OFF);
  for (i=0;i<*ndata;i++) index_array[i]++; 
}





void derefoffset_(trans_table,index_array,local,ndata)
     intPtr trans_table, index_array, local,ndata;
{ 
  int i;

  for (i=0;i<*ndata;i++) index_array[i]--; 
  derefoffset(get_table_ptr(*trans_table),index_array,local,*ndata,OFF);
  for (i=0;i<*ndata;i++) index_array[i]++,local[i]++; 
}




void remap_table_(oldt, newIndex, nindex)
     intPtr oldt, newIndex, nindex;
{
  int i;

  for (i=0;i<*nindex;i++) newIndex[i]--; 
  remap_table(get_table_ptr(*oldt), newIndex, *nindex);
  for (i=0;i<*nindex;i++) newIndex[i]++; 
}



void derefprocadaptive_(table,index,proc,ndata,aflag)
     intPtr table, index, proc, ndata, aflag;
{
  TTABLE  *ttab;
  int	i;

#ifdef COMPARE
  int *test, bugs;
  int   pg, of;
  DSTABLE *dstab;
#endif

  ttab = get_table_ptr(*table);

#ifdef COMPARE
  if (mynode()==0) printf("\n A=%d R=%f", *aflag, ttab->tabptr->repFactor);
  gsync();
#endif
  
  for(i=0; i < *ndata ;i++) index[i]--;
  derefproc(ttab,index,proc,*ndata,*aflag);
  for(i=0; i < *ndata ;i++) index[i]++;

#ifdef COMPARE
  test = (int*)malloc(sizeof(int)*(*ndata));
  derefproc(ttab,index,test,*ndata,0);
  for (bugs=i=0; i<*ndata; i++)
    if (test[i]!=proc[i]) 
      {
	bugs++;
	proc[i] = test[i];
      }
  free(test);

  if (bugs)
    printf("\n %d Mismatches @%d", bugs, mynode());
  gsync();
#endif
}



void derefoffsetadaptive_(table,index,offset,ndata,aflag)
     intPtr table, index, offset, ndata, aflag;
{
  int	i;
  
  for(i=0; i < *ndata ;i++) index[i]--;
  derefoffset(get_table_ptr(*table),index,offset,*ndata,*aflag);
  for(i=0; i < *ndata ;i++) index[i]++, offset[i]++;
}



void derefadaptive_(table,index,offset,proc,ndata,aflag)
     intPtr table, index, offset, proc,ndata, aflag;
{
  int	i;
  
  for(i=0; i < *ndata ;i++) index[i]--;
  _dereference(get_table_ptr(*table),index,offset,proc,*ndata,*aflag);
  for(i=0; i < *ndata ;i++) index[i]++, offset[i]++;
}



#ifdef COLLSTAT
static double *timings = NULL;
static int  *pageMoves = NULL;
static int  *pgcnt     = NULL;
static int  *tmcnt     = NULL; 
static int  __interval = 0;

void ctrsetup_(interval)
     intPtr interval;
{
  int i;

  timings   = (double*) malloc(*interval*sizeof(double));
  pageMoves = (int*) malloc(*interval*sizeof(int));
  pgcnt     = (int*) malloc(*interval*sizeof(int));
  tmcnt     = (int*) malloc(*interval*sizeof(int));
  __interval = *interval;

  for (i=0; i<*interval; i++)
    {
      timings[i] = 0.0;
      pgcnt[i] = tmcnt[i] = pageMoves[i] = 0;
    }
}


void timingpoint_(point, timeval)
     int    *point;
     double *timeval;
{
  timings[*point] += *timeval;
  tmcnt[*point]++;
}

extern int pageIn;

void pagepoint_(point)
     int    *point;
{
  pageMoves[*point] += pageIn;
  pgcnt[*point]++;
}


void ctrsummary_()
{
  double dtmp,dmax,dmin,dsum,davg,iavg;
  int    itmp,imax,imin,isum;
  int    tcnt, pcnt,i;

  fflush(stdout);
  gsync();

  if (mynode()==0)
    {
      printf("\n\n  Timings over Remap Interval");
      printf("\n -----------------------------");
      printf("\nTSt    Max     Min     Avg/iter");
      fflush(stdout);
    }
  for (i=0; i<__interval; i++)
    {
      dmax = dmin = dsum = davg = timings[i];
      tcnt = tmcnt[i];

      gdsum(&dsum, 1, &dtmp);
      gdhigh(&dmax, 1, &dtmp);
      gdlow(&dmin, 1, &dtmp);
      gisum(&tcnt, 1, &itmp);
      davg = dsum / (double)tcnt;

      if (mynode()==0)
	printf("\n %d  %lf  %lf  %lf", i, dmax, dmin, davg), fflush(stdout);;
    }

  if (mynode()==0)
    {
      printf("\n\n  Page Counts over Remap Interval");
      printf("\n -----------------------------");
      printf("\nTSt Sum  Max  Min   Avg/iter");
      fflush(stdout);
    }

  for (i=0; i<__interval; i++)
    {
      imax = imin = isum = iavg = pageMoves[i];
      pcnt = pgcnt[i];

      gisum(&isum, 1, &itmp);
      gihigh(&imax, 1, &itmp);
      gilow(&imin, 1, &itmp);
      gisum(&pcnt, 1, &itmp);
      iavg = (double)isum / (double)pcnt;

      if (mynode()==0)
	printf("\n %d %+4d %+4d %+4d %+10lf %d", i, isum, imax, imin, iavg,pcnt), fflush(stdout);
    }
}


#endif



